@@ -2,6 +2,7 @@ from datetime import datetime
 from uuid import uuid4
 import logging
 import numbers
+import atexit
 
 from dateutil.tz import tzutc
 from six import string_types
@@ -34,6 +35,13 @@ class Client(object):
         self.debug = debug
         self.send = send
 
+        # On program exit, allow the consumer thread to exit cleanly.
+        # This prevents exceptions and a messy shutdown when the interpreter is
+        # destroyed before the daemon thread finishes execution. However, it
+        # is *not* the same as flushing the queue! To guarantee all messages
+        # have been delivered, you'll still need to call flush().
+        atexit.register(self.join)
+
         if debug:
             self.log.setLevel(logging.DEBUG)
 
@@ -41,8 +49,11 @@ class Client(object):
         if send:
             self.consumer.start()
 
-    def identify(self, user_id=None, traits={}, context={}, timestamp=None,
-                 anonymous_id=None, integrations={}):
+    def identify(self, user_id=None, traits=None, context=None, timestamp=None,
+                 anonymous_id=None, integrations=None):
+        traits = traits or {}
+        context = context or {}
+        integrations = integrations or {}
         require('user_id or anonymous_id', user_id or anonymous_id, ID_TYPES)
         require('traits', traits, dict)
 
@@ -58,9 +69,11 @@ class Client(object):
 
         return self._enqueue(msg)
 
-    def track(self, user_id=None, event=None, properties={}, context={},
-              timestamp=None, anonymous_id=None, integrations={}):
-
+    def track(self, user_id=None, event=None, properties=None, context=None,
+              timestamp=None, anonymous_id=None, integrations=None):
+        properties = properties or {}
+        context = context or {}
+        integrations = integrations or {}
         require('user_id or anonymous_id', user_id or anonymous_id, ID_TYPES)
         require('properties', properties, dict)
         require('event', event, string_types)
@@ -78,8 +91,10 @@ class Client(object):
 
         return self._enqueue(msg)
 
-    def alias(self, previous_id=None, user_id=None, context={}, timestamp=None,
-              integrations={}):
+    def alias(self, previous_id=None, user_id=None, context=None,
+              timestamp=None, integrations=None):
+        context = context or {}
+        integrations = integrations or {}
         require('previous_id', previous_id, ID_TYPES)
         require('user_id', user_id, ID_TYPES)
 
@@ -94,8 +109,11 @@ class Client(object):
 
         return self._enqueue(msg)
 
-    def group(self, user_id=None, group_id=None, traits={}, context={},
-              timestamp=None, anonymous_id=None, integrations={}):
+    def group(self, user_id=None, group_id=None, traits=None, context=None,
+              timestamp=None, anonymous_id=None, integrations=None):
+        traits = traits or {}
+        context = context or {}
+        integrations = integrations or {}
         require('user_id or anonymous_id', user_id or anonymous_id, ID_TYPES)
         require('group_id', group_id, ID_TYPES)
         require('traits', traits, dict)
@@ -113,8 +131,12 @@ class Client(object):
 
         return self._enqueue(msg)
 
-    def page(self, user_id=None, category=None, name=None, properties={},
-             context={}, timestamp=None, anonymous_id=None, integrations={}):
+    def page(self, user_id=None, category=None, name=None, properties=None,
+             context=None, timestamp=None, anonymous_id=None,
+             integrations=None):
+        properties = properties or {}
+        context = context or {}
+        integrations = integrations or {}
         require('user_id or anonymous_id', user_id or anonymous_id, ID_TYPES)
         require('properties', properties, dict)
 
@@ -137,8 +159,12 @@ class Client(object):
 
         return self._enqueue(msg)
 
-    def screen(self, user_id=None, category=None, name=None, properties={},
-               context={}, timestamp=None, anonymous_id=None, integrations={}):
+    def screen(self, user_id=None, category=None, name=None, properties=None,
+               context=None, timestamp=None, anonymous_id=None,
+               integrations=None):
+        properties = properties or {}
+        context = context or {}
+        integrations = integrations or {}
         require('user_id or anonymous_id', user_id or anonymous_id, ID_TYPES)
         require('properties', properties, dict)
 
@@ -184,20 +210,21 @@ class Client(object):
         msg = clean(msg)
         self.log.debug('queueing: %s', msg)
 
-        if self.queue.full():
+        try:
+            self.queue.put(msg, block=False)
+            self.log.debug('enqueued %s.', msg['type'])
+            return True, msg
+        except queue.Full:
             self.log.warn('analytics-python queue is full')
             return False, msg
 
-        self.queue.put(msg)
-        self.log.debug('enqueued ' + msg['type'] + '.')
-        return True, msg
-
     def flush(self):
         """Forces a flush from the internal queue to the server"""
         queue = self.queue
         size = queue.qsize()
         queue.join()
-        self.log.debug('successfully flushed {0} items.'.format(size))
+        # Note that this message may not be precise, because of threading.
+        self.log.debug('successfully flushed about %s items.', size)
 
     def join(self):
         """Ends the consumer thread once the queue is empty. Blocks execution until finished"""
@@ -205,7 +232,6 @@ class Client(object):
         self.consumer.join()
 
 
-
 def require(name, field, data_type):
     """Require that the named `field` has the right `data_type`"""
     if not isinstance(field, data_type):
